<!DOCTYPE html>
<html lang="{{ site.lang | default: " en-us" }}"="en-US" }}""="}}" "">
    <head>

        {% if site.google_analytics %}
            <script async="async" src="https://www.googletagmanager.com/gtag/js?id={{ site.google_analytics }}"></script>
            <script>
                window.dataLayer = window.dataLayer || [];
                function gtag() {
                    dataLayer.push(arguments);
                }
                gtag('js', new Date());
                gtag('config', '{{ site.google_analytics }}');
            </script>
        {% endif %}
        <meta charset="UTF-8">

        {% seo %}
        <meta name="viewport" content="width=device-width, initial-scale=1">
        <meta name="theme-color" content="#157878">
        <meta name="apple-mobile-web-app-status-bar-style" content="black-translucent">
        <link rel="stylesheet" href="{{ '/assets/css/style.css?v=' | append: site.github.build_revision | relative_url }}">
    </head>
    <body>
        <a style="display: none;" id="skip-to-content" href="#content">Zum Inhalt springen.</a>

        <header class="page-header" role="banner">
            <h1 class="project-name">KMUHelper</h1>
            <h2 class="project-tagline">{{ site.description | default: site.github.project_tagline }}</h2>
            <a href="{{ '/' | relative_url }}" class="btn">Home</a>
            <a href="{{ '/manual/' | relative_url }}" class="btn">Handbuch</a>
            {% if site.github.is_project_page %}
                <a href="{{ site.github.repository_url }}" class="btn">Auf GitHub ansehen</a>
            {% endif %}
            {% if site.show_downloads %}
                <a href="{{ site.github.zip_url }}" class="btn">Download .zip</a>
                <a href="{{ site.github.tar_url }}" class="btn">Download .tar.gz</a>
            {% endif %}
        </header>

        <main id="content" class="main-content" role="main">
            {{ content }}

            <footer class="site-footer">
                {% if site.github.is_project_page %}
                    <span class="site-footer-owner">
                        <a href="{{ site.github.repository_url }}">{{ site.github.repository_name }}</a>
                        wird von
                        <a href="{{ site.github.owner_url }}">{{ site.github.owner_name }}</a>
                        unterhalten.</span>
                {% endif %}
                <span class="site-footer-credits">Diese Seite wurde von
                    <a href="https://pages.github.com">GitHub Pages</a>
                    generiert.</span>
            </footer>
        </main>
    </body>
</html>