{% extends 'admin/kmuhelper/change_form.html' %}

{% block branding %}
<h1 id="site-name"><a target="_top" href="{% url 'kmuhelper:app-main' %}#home">KMUHelper App</a></h1>
{% endblock %}

{% block userlinks %}{% endblock %}

{% block breadcrumbs %}{% endblock %}

{% block object-tools %}{% endblock %}
