{% extends "admin/delete_confirmation.html" %}
{% load static %}

{% block title %}{{ title }} | KMUHelper{% endblock %}

{% block branding %}
    <h1 id="site-name"><a id="kmuhelper_admin" href="{% url 'admin:app_list' app_label='kmuhelper' %}">KMUHelper Admin</a> / <a id="kmuhelper_app" target="_top" href="{% url 'kmuhelper:app-main' %}">App</a></h1>
{% endblock %}

{% block extrastyle %}
    {{ block.super }}
    <link rel="stylesheet" type="text/css" href="{% static "admin/kmuhelper/css/style.css" %}">
{% endblock %}

{% block extrahead %}
    {{ block.super }}
    <link rel="shortcut icon" type="image/x-icon" href="{% static 'kmuhelper/favicon.ico' %}">
    <link rel="apple-touch-icon" href="{% static 'kmuhelper/favicon.ico' %}">
{% endblock %}
