<h1>Guten Tag!</h1>

<p>Im Anhang finden Sie Ihre Rechnung zur Bestellung Nr. {{ id }}{% if woocommercedata %} (Online #{{ woocommerceid }}){% endif %}</p>

{% if trackingdata %}
    <p>Sie können die Bestellung über <a href="{{ trackinglink }}">diesen Link</a> verfolgen, sobald Sie von der Post verarbeitet wurde.</p>
{% endif %}
