{% extends 'admin/kmuhelper/change_list.html' %}

{% block branding %}
<h1 id="site-name"><a target="_top" href="{% url 'kmuhelper:app-main' %}#home">KMUHelper App</a></h1>
{% endblock %}

{% block userlinks %}{% endblock %}

{% block breadcrumbs %}{% endblock %}

{% block extrahead %}
    {{ block.super }}
    <script>
        'use strict'
        var $ = django.jQuery

        $(document).ready(function () {
            // Add Buttonrow to top too
            var saveButtonRow = $('.paginator')[0]
            var resultsDiv = $('#changelist-form > div.results')[0]

            if (saveButtonRow && resultsDiv) {
                $(saveButtonRow).clone().insertBefore(resultsDiv);
            }
        })
    </script>
{% endblock %}
