{% extends 'admin/kmuhelper/change_list.html' %}

{% block object-tools-items %}
    <li><a href="{% url 'kmuhelper:wc-auth-start' %}">WooCommerce verbinden</a></li>
    {{ block.super }}
{% endblock %}
