{% extends 'admin/kmuhelper/change_list.html' %}

{% block object-tools-items %}
    <!-- SweetAlert -->
    <script src="https://unpkg.com/sweetalert/dist/sweetalert.min.js"></script>
    <script>
        function laden() {
            swal({
                title: "Bitte warten...",
                text: "Dieser Prozess kann unter Umständen etwas dauern.\n\nBitte versuche nicht, diesen Prozess abzubrechen!\n\nFalls du eine Fehlermeldung erhältst, lade bitte die Seite neu!\n\n(Dies könnte je nach Menge auch mehrmals der Fall sein.)",
                icon: "info",
                buttons: false,
                closeOnClickOutside: false,
                closeOnEsc: false
            });
        };
    </script>
    <li>
        <a onclick="laden();" href="{% url 'kmuhelper:wc-import-customers' %}" class="addlink">Kunden von WooCommerce importieren</a>
    </li>
    {{ block.super }}
{% endblock %}