{% extends 'admin/kmuhelper/change_form.html' %}

{% block object-tools %}
  <ul class="object-tools">
      {% if change %}{% if not is_popup %}
          {% block object-tools-items %}
              {% if original.woocommerceid %}
              <li><a href="{% url 'kmuhelper:wc-update-customer' object_id %}">Von WooCommerce importieren und ersetzen</a></li>
              {% endif %}
              {% if original.email %}{% if not original.registrierungsemail_gesendet %}
              <li><a href="{% url 'kmuhelper:kunde-email-registriert' object_id %}">Registrierungsemail senden</a></li>
              {% endif %}{% endif %}
              {{ block.super }}
          {% endblock %}
      {% endif %}{% endif %}

      <li><a href="#id_lieferadresse_vorname" onclick="copyform();">Rechnungsadresse -> Lieferadresse</a></li>
      <script>
      function copyform(){
          document.getElementById("id_lieferadresse_vorname").value = document.getElementById("id_rechnungsadresse_vorname").value;
          document.getElementById("id_lieferadresse_nachname").value = document.getElementById("id_rechnungsadresse_nachname").value;
          document.getElementById("id_lieferadresse_firma").value = document.getElementById("id_rechnungsadresse_firma").value;
          document.getElementById("id_lieferadresse_adresszeile1").value = document.getElementById("id_rechnungsadresse_adresszeile1").value;
          document.getElementById("id_lieferadresse_adresszeile2").value = document.getElementById("id_rechnungsadresse_adresszeile2").value;
          document.getElementById("id_lieferadresse_ort").value = document.getElementById("id_rechnungsadresse_ort").value;
          document.getElementById("id_lieferadresse_kanton").value = document.getElementById("id_rechnungsadresse_kanton").value;
          document.getElementById("id_lieferadresse_plz").value = document.getElementById("id_rechnungsadresse_plz").value;
          document.getElementById("id_lieferadresse_land").value = document.getElementById("id_rechnungsadresse_land").value;
      }
      </script>
  </ul>
{% endblock %}
