---
---

@import "{{ site.theme }}";

:root {
  --kmuhelper-darkest: #880044;
  --kmuhelper-dark: #AA1155;
  --kmuhelper-default: #DD1155;
  --kmuhelper-light: #F6C400;
  --kmuhelper-lightest: #FFEE88;
}

body {
    scroll-behavior: smooth;
}

.page-header {
    background-color: var(--kmuhelper-darkest);
    background-image: linear-gradient(120deg, var(--kmuhelper-darkest), var(--kmuhelper-light));
}

.main-content h1, .main-content h2, .main-content h3, .main-content h4, .main-content h5, .main-content h6 {
    color: var(--kmuhelper-default);
}

a {
    color: var(--kmuhelper-dark);
}

img {
    border-radius: 5px;
    margin: 5px;
    -webkit-box-shadow: 0px 0px 10px 2px rgba(0,0,0,0.5);
    -moz-box-shadow: 0px 0px 10px 2px rgba(0,0,0,0.5);
    box-shadow: 0px 0px 10px 2px rgba(0,0,0,0.5);
}
